/**
 * Interface for server side communication
 **/
jsp.api = (function() {
	'use strict';
	var pluginName = location.pathname.split("/")[2],
		dunePhpUrl = '/cgi-bin/plugins/' + pluginName + '/do_php/',
		proxyUrl = {
			json: dunePhpUrl + 'curl-api-proxy.php?path=',
			xml: dunePhpUrl + 'xml_parser.php?path='
		},

		CONFIG_URL = 'http://127.0.0.1/plugins/' + pluginName + '/config.xml',
		CONFIG_TYPE = 'xml',

		channels = {},
		channelsArr = [],
		options;

	window.alert = function(string) {
		document.getElementById('debug').innerHTML += string + '<br>';
	};

	function isEpgEnable() {
		return options.epg_enable === "true";
	}

	function loadConfig(params) {
		var url = proxyUrl[CONFIG_TYPE] + CONFIG_URL;
		jsp.ajax.get(url, function(res) {
			options = res;
			if (params.callbackSuccess) {
				params.callbackSuccess();
			}
		}, params.callbackError);
	}

	function updatePrograms(items) {
		var i, len, item;

		for (i = 0, len = items.length; i < len; i++) {
			item = items[i];
			item.startDate = new Date(item.start_time * 1000);
			item.stopDate = new Date(item.stop_time * 1000);
			item.date = jsp.dateToString(item.startDate);
			item.time = jsp.timeToString(item.startDate);
		}
	}

	function loadChannelProgram(params) {
		var channel = channels[params.channel],
			url = proxyUrl[channel.program_type] + channel.program_url;

		jsp.ajax.get(url, function(res) {
			updatePrograms(res.programs_item);
			channels[params.channel].programs = res.programs_item;

			params.callbackSuccess && params.callbackSuccess();
		}, params.callbackError);
	}

	function loadPrograms(params) {
		if (!isEpgEnable()) {
			params.callbackSuccess();
			return;
		}

		var loadedChannelsCount = 0,
			i,
			len,
			item;

		function loadCallback() {
			loadedChannelsCount++;
			if (loadedChannelsCount === len && params.callbackSuccess) {
				params.callbackSuccess();
			}
		}

		for (i = 0, len = channelsArr.length; i < len; i++) {
			item = channelsArr[i];
			if (item.program_url) {
				loadChannelProgram({
					channel: channelsArr[i].id,
					callbackSuccess: loadCallback,
					callbackError: loadCallback
				});
			} else {
				loadCallback();
			}
		}
	}

	function loadChannelsInfo(params) {
		var url = proxyUrl[options.channel_type] + options.channel_url;

		jsp.ajax.get(url, function(res) {
			var i, len, item;
			channelsArr = res.tv_channels.tv_channel;

			for (i = 0, len = channelsArr.length; i < len; i++) {
				item = channelsArr[i];
				item.number = i;
				channels[item.id] = item;
			}

			params.callbackSuccess && params.callbackSuccess();
		}, params.callbackError);
	}

	function loadData(params) {
		loadConfig({
			callbackSuccess: function() {
				loadChannelsInfo({
					callbackSuccess: function () {
						loadPrograms(params)
					},
					callbackError: params.callbackError
				});
			},
			callbackError: params.callbackError
		});
	}

	function savePlayChannel(params) {
		jsp.duneapi.setUserSetting(pluginName + "playChannel", params.channel);
	}

	function imagePreload() {
		var img = document.createElement("img"),
			index = 0;

		function changeSrc() {
			var channel = channelsArr[index];

			if (channel) {
				setTimeout(function() {
					img.src = channel.icon_url;
				}, 100);
			}
			index++;
		}

		img.onload = changeSrc;
		changeSrc();
	}

	function getProgramsByDate(channel, dateTime) {
		var programs = channels[channel].programs || [],
			datePrograms = [],
			date = jsp.dateToString(dateTime),
			item,
			i,
			len;

		for (i = 0, len = programs.length; i < len; i++) {
			item = programs[i];
			if (item.date === date) {
				datePrograms.push(item);
			}
		}

		return datePrograms;
	}

	function findProgramByTime(channel, dateTime) {
		var programs = channels[channel].programs || [],
			i,
			len,
			item;

		for (i = 0, len = programs.length; i < len; i++) {
			item = programs[i];

			if (item.startDate <= dateTime && dateTime < item.stopDate) {
				return item;
			}
		}

		return null;
	}

	function nextProgram(channel, prevProgram) {
		var programs = channels[channel].programs || [],
			programIndex = programs.indexOf(prevProgram);

		return programs[programIndex + 1] || null;
	}

	function getChannelInfo(channel) {
		return channels[channel] || null;
	}

	function getChannelInfoByNumber(number) {
		return channelsArr[number] || null;
	}

	function getFirstChannel() {
		return channelsArr[0] || null;
	}

	function getLastChannel() {
		return channelsArr[channelsArr.length - 1] || null;
	}

	function getChannelsList() {
		return channelsArr;
	}

	function getPlayingChannelId() {
		var result = jsp.duneapi.getUserSetting(pluginName + "playChannel") || getFirstChannel().id;
		return result;
	}

	function getExitPluginKey() {
		return options.exit_plugin_key;
	}

	function getChannelsCount() {
		return channelsArr.length;
	}

	return {
		getProgramsByDate: getProgramsByDate,
		findProgramByTime: findProgramByTime,
		getChannelInfo: getChannelInfo,
		getChannelInfoByNumber: getChannelInfoByNumber,
		nextProgram: nextProgram,
		loadData: loadData,
		getChannelsList: getChannelsList,
		getFirstChannel: getFirstChannel,
		getLastChannel: getLastChannel,
		getPlayingChannelId: getPlayingChannelId,
		savePlayChannel: savePlayChannel,
		getExitPluginKey: getExitPluginKey,
		isEpgEnable: isEpgEnable,
		imagePreload: imagePreload,
		getChannelsCount: getChannelsCount
	};
})();
