/**
 * Perform an asynchronous HTTP (Ajax) request.
 */
jsp.ajax = (function () {
    "use strict";
    var requestTimeout =  5000;

    function defaultErrorCB() {
        jsp.duneapi.log("IPTV_DEMO: server not response");
    }
    function defaultSuccessCB() {}

    function parseResponse(responseText, successCB, errorCB) {
        var responseObject = null;
        try {
            responseObject = JSON.parse(responseText);
        } catch (e) {
            errorCB("Incorrect response");
        }
        successCB(responseObject);
    }

    function getRequest(url, success, error) {
        var errorCB = error || defaultErrorCB,
            successCB = success || defaultSuccessCB,
            errorTimer,
            httpRequest;

        function showError() {
            errorCB("Communication error");
        }

        errorTimer = setTimeout(function () {
            httpRequest.abort();
            showError();
        }, requestTimeout);

        httpRequest = new XMLHttpRequest();
        httpRequest.open('GET', url);

        httpRequest.onreadystatechange = function () {
            if (httpRequest.readyState === 4) {
                clearTimeout(errorTimer);

                if (httpRequest.status !== 200) {
                    httpRequest.abort();
                    showError();
                } else {
                    parseResponse(httpRequest.responseText, successCB, errorCB);
                }
            }
        };
        httpRequest.send();

        return httpRequest;
    }

    return {
        get: getRequest
    };
})();