/**
 * Auxiliary functions for working with the DOM, time
 **/
jsp.DEFAULT_KEY_EVENT = "keydown";
jsp.DAY_NAMES = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
jsp.MILISEC_IN_DAY = 86400000;
if (!("classList" in document.documentElement) && Object.defineProperty && typeof HTMLElement !== 'undefined') {
    Object.defineProperty(HTMLElement.prototype, 'classList', {
        get: function() {
            var self = this;
            function update(fn) {
                return function(value) {
                    var classes = self.className.split(/\s+/),
                        index = classes.indexOf(value);

                    fn(classes, index, value);
                    self.className = classes.join(" ");
                }
            }

            var ret = {
                add: update(function(classes, index, value) {
                    ~index || classes.push(value);
                }),

                remove: update(function(classes, index) {
                    ~index && classes.splice(index, 1);
                }),

                toggle: update(function(classes, index, value) {
                    ~index ? classes.splice(index, 1) : classes.push(value);
                }),

                contains: function(value) {
                    return !!~self.className.split(/\s+/).indexOf(value);
                },

                item: function(i) {
                    return self.className.split(/\s+/)[i] || null;
                }
            };

            Object.defineProperty(ret, 'length', {
                get: function() {
                    return self.className.split(/\s+/).length;
                }
            });

            return ret;
        }
    });
}
/* --------------------------------- DOM ----------------------------- */
/**
 * Find element by id from DOM
 */
jsp.byId = function (id) {
    return document.getElementById(id);
};

/**
 * Find element by className from DOM or element
 */
jsp.byClass = function (className, element) {
    var obj = element || document;
    return obj.getElementsByClassName(className)[0];
};

jsp.hide = function (element) {
    element.classList.add("hidden");
};

jsp.show = function (element) {
    element.classList.remove("hidden");
};

jsp.changeVisible = function (element, visible) {
    if (visible) {
        jsp.show(element);
    } else {
        jsp.hide(element);
    }
};

jsp.toggle = function (element) {
    if (element.classList.contains("hidden")) {
        jsp.show(element);
    } else {
        jsp.hide(element);
    }
};

jsp.isVisible = function (element) {
    return !element.classList.contains("hidden");
};

jsp.toggleClass = function (element, className) {
    if (element.classList.contains(className)) {
        element.classList.remove(className);
    } else {
        element.classList.add(className);
    }
};

/* --------------------------------- TIME ----------------------------- */

jsp.timeToString = function (date) {
    var h = date.getHours(),
        m = date.getMinutes();

    return (h < 10 ? "0" + h : h) + ":" + (m < 10 ? "0" + m : m);
};

jsp.timeWithSecToString = function (date) {
    var h = date.getHours(),
        m = date.getMinutes(),
        s = date.getSeconds();

    return (h < 10 ? "0" + h : h) + ":" + (m < 10 ? "0" + m : m) + ":" + (s < 10 ? "0" + s : s);
};

jsp.dateToString = function (date) {
    var y = date.getFullYear(),
        m = date.getMonth() + 1,
        d = date.getDate();

    return y + "-" + (m < 10 ? "0" + m : m) + "-" + (d < 10 ? "0" + d : d);
};

jsp.dateToDayName = function (date) {
    var day = date.getDay();

    return jsp.DAY_NAMES[day];
};

jsp.addDaysToDate = function (date, days) {
    return new Date(date.getTime() + days * jsp.MILISEC_IN_DAY);
};