/**
 * TV player and programs for a channels.
 */
jsp.view = (function () {
	var CLOCK_UPDATE_DURATION = 1000,
		HIDE_INFO_DURATION = 5000,
		PROGRAM_NOT_FIND = "TV listings not available for this channel",

		isEpgEnable, viewDiv, updateInfoTimer = null,
		hideInfoTimer, renderProgramsTimer, channel,
		programsDate, programsChannel, needFocusToProgram,

		videoTag, infoTag, currentProgramTag, channelTag, clockTag,
		channelsTag, channelsList, channelsScrollBtn, programsTag,
		programsTitle, programsList, nextArrow, prevArrow, programsScrollBtn;


	function centerEPG() {
		if (!isEpgEnable) {
			var MAX_COUNT = 9,
				SCREEN_HEIGHT = 660,
				HEADER_HEIGHT = 52,
				ITEM_HEIGHT = 65,
				chCount = jsp.api.getChannelsCount();

			if (chCount > MAX_COUNT) {
				chCount = MAX_COUNT;
			}
			channelsTag.style.marginTop = (SCREEN_HEIGHT - HEADER_HEIGHT - ITEM_HEIGHT * chCount) / 2 + "px";
		}
	}

	function bindVideoEvents() {
		jsp.duneapi.setPlaybackEventCallback(function (previousState, currentState) {
			switch (currentState) {
			case jsp.duneapi.PLAYBACK_STATE_INITIALIZING:
				document.body.classList.add("start-load");
				break;
			case jsp.duneapi.PLAYBACK_STATE_PLAYING:
				document.body.classList.add("play-video");
				document.body.classList.remove("start-load");
				break;
			default:
				break;
			}
		});
	}

	function generateChannelsHtmlForMenu() {
		var channels = jsp.api.getChannelsList(),
			html = '';

		for (var i = 0, len = channels.length, item; i < len; i++) {
			item = channels[i];
			html +=
				'<div id="' + item.id + '" tabindex="1" data-right="focusToProgram" data-enter="openChannel" data-up="channelsListUp" data-down="channelsListDown" class="item">' +
					'<div class="numb">' + (item.number + 1) + '</div>' +
					(item.icon_url ? '<div class="img" style="background-image: url(\'' + item.icon_url + '\');"></div>' : "") +
					'<div class="caption">' + item.caption + '</div>' +
					'</div>';
		}
		return html;
	}

	function init() {
		isEpgEnable = jsp.api.isEpgEnable();
		viewDiv = jsp.byId("main");
		videoTag = jsp.byClass("video", viewDiv);
		infoTag = jsp.byClass("info", viewDiv);
		currentProgramTag = jsp.byClass("play-programs", infoTag);
		channelTag = jsp.byClass("play-channel", infoTag);
		clockTag = jsp.byClass("clock", infoTag);
		channelsTag = jsp.byClass("channels", viewDiv);
		channelsList = jsp.byClass("list", channelsTag);
		channelsScrollBtn = jsp.byClass("scroll-btn", channelsTag);
		programsTag = jsp.byClass("programs", viewDiv);
		programsTitle = jsp.byClass("title", programsTag);
		programsList = jsp.byClass("list", programsTag);
		nextArrow = jsp.byClass("next", programsTag);
		prevArrow = jsp.byClass("prev", programsTag);
		programsScrollBtn = jsp.byClass("scroll-btn", programsTag);

		if (!isEpgEnable) channelsTag.classList.add("big");
		channelsList.innerHTML = generateChannelsHtmlForMenu();

		bindVideoEvents();

		jsp.duneapi.setColorKey(0xFF0000);

		// bind keydown event listener
		viewDiv.addEventListener("keydown", keyListener);

		// move channels menu to center od screen when EPG is disable
		centerEPG();

		//auto update the clock
		(function() {
			var dateTime = new Date();
			clockTag.innerHTML = jsp.timeToString(dateTime);
			setTimeout(arguments.callee, CLOCK_UPDATE_DURATION);
		})();

		// autoplay previous channel
		render(jsp.api.getPlayingChannelId());
	}

	function render(channelId) {
		channel = jsp.api.getChannelInfo(channelId);
		jsp.duneapi.play(channel.streaming_url);
		// document.getElementById('video').src = channel.streaming_url;

		jsp.hide(infoTag);
		toggleChannelInfo();
		markCurrentPlayChannel(channelId);
		showCurrentChannelInfo();
		hideInfoTimer = setTimeout(toggleChannelInfo, HIDE_INFO_DURATION);
		jsp.api.savePlayChannel({channel: channelId});
		videoTag.focus();
	}

	/* ------------------------------- INFO ABOUT CURRENT CHANNEL --------------------------- */

	function showCurrentChannelInfo() {
		updateChannelInfo(channel, new Date());
	}

	function updateChannelInfo(channel, dateTime) {
		var currPlay = jsp.api.findProgramByTime(channel.id, dateTime),
			nextPlay = jsp.api.nextProgram(channel.id, currPlay),
			livePlay = jsp.api.findProgramByTime(channel.id, new Date()),
			status = (currPlay === livePlay ? "LIVE" : "Airtime"),
			html = '';

		channelTag.innerHTML =
			'<div class="numb">' + (channel.number + 1) + '</div>' +
				(channel.icon_url ? '<div class="img" style="background-image: url(\'' + channel.icon_url + '\');"></div>' : "") +
				'<div class="caption">' + channel.caption + '</div>';

		if (isEpgEnable) {
			if (currPlay) {
				html =
					'<div class="time">' + status + ' ' + currPlay.time + ' - ' + jsp.timeToString(currPlay.stopDate) + '</div>' +
					'<div class="curr-play">' + (currPlay.caption ? currPlay.caption + '. ' : '') + currPlay.description + '</div>';

				if (nextPlay) {
					html +=
						'<div class="next-time">NEXT ' + nextPlay.time + ' - ' + jsp.timeToString(nextPlay.stopDate) + '</div>' +
						'<div class="next-play">' + (nextPlay.caption ? nextPlay.caption + '. ' : '') + nextPlay.description + '</div>';
				}
			} else {
				html = PROGRAM_NOT_FIND;
			}

			currentProgramTag.innerHTML = html;
		}
	}

	function toggleChannelInfo() {
		clearTimeout(hideInfoTimer);
		if (jsp.isVisible(channelsTag)) hideChannels();
		jsp.toggle(infoTag);
	}

	/* ------------------------ CHANNELS LIST ------------------------- */

	function showChannels() {
		clearTimeout(hideInfoTimer);
		jsp.changeVisible(infoTag, isEpgEnable);
		jsp.changeVisible(programsTag, isEpgEnable);
		jsp.show(channelsTag);

		updateChannelsScroll();
		loadPrograms(channel.id);
	}

	function hideChannels() {
		jsp.hide(channelsTag);
		jsp.hide(programsTag);
		jsp.hide(infoTag);
		videoTag.focus();
	}

	function openChannel() {
		var channelId = document.activeElement.id;
		if (channelId != channel.id) {
			render(channelId);
		} else {
			videoTag.focus();
			hideChannels();
		}
	}

	function changeChannel(direction) {
		var newChannel = jsp.api.getChannelInfoByNumber(channel.number+direction);

		if (!newChannel) {
			if (direction === 1) {
				newChannel = jsp.api.getFirstChannel();
			} else {
				newChannel = jsp.api.getLastChannel();
			}
		}

		if (newChannel) render(newChannel.id);
	}

	function changeChannelUp() {
		changeChannel(+1);
	}

	function changeChannelDown() {
		changeChannel(-1);
	}

	function markCurrentPlayChannel(id) {
		var prev = jsp.byClass("play-now", channelsTag),
			next = jsp.byId(id);

		if (prev) prev.classList.remove("play-now");
		next.classList.add("play-now");
	}

	/* ------------------------ PROGRAMS LIST ------------------------- */

	function loadPrograms(channelId) {
		clearTimeout(renderProgramsTimer);

		var newChannel = jsp.byId(channelId),
			prevChannel = jsp.byClass("active", channelsTag),
			date = new Date();

		if (prevChannel) prevChannel.classList.remove("active");
		newChannel.classList.add("active");

		newChannel.focus();

		renderProgramsTimer = setTimeout(function(){
			renderProgramsTimer = null;
			renderPrograms(channelId, date);
			updateChannelInfo(jsp.api.getChannelInfo(channelId), date);
		}.bind(this), 500);
	}

	function renderPrograms (channelId, date) {
		var items = jsp.api.getProgramsByDate(channelId, date),
			active = jsp.api.findProgramByTime(channelId, new Date()),
			item, html = '';

		if (!items.length && programsChannel == channelId) {
			return;
		}

		programsDate = date;
		programsChannel = channelId;
		programsTitle.innerHTML = jsp.dateToDayName(date) + ', ' + jsp.dateToString(date);
		programsList.scrollTop = 0;

		for (var i = 0, len = items.length; i < len; i++) {
			item = items[i];
			html +=
				'<div tabindex="1" data-index="' + i + '" data-enter="hideChannels" data-next="programsNextDay" data-prev="programsPrevDay" data-up="' + (!i ? "programsPrevDay" : "prevProgram") + '" data-down="' + (i == len-1 ? "programsNextDay" : "nextProgram") + '" data-left="focusToChannel" data-time="' + item.startDate.getTime() + '" class="item ' + (item === active? 'active' : '') + '">' +
					'<div class="time">' + item.time + '</div>' +
					'<div class="description">' + (item.caption ? item.caption + '. ' : '') + item.description + '</div>' +
				'</div>';
		}
		if (!items.length) html += PROGRAM_NOT_FIND;
		programsList.innerHTML = html;

		updateProgramsPosition();
		updateProgramsScrollBar();
		updateProgramsArrow(channelId, date);

		if(needFocusToProgram){
			needFocusToProgram = false;
			focusToProgram();
		}
	}

	function updateProgramsArrow(channelId, date) {
		var nextDayProgram = jsp.api.getProgramsByDate(channelId, jsp.addDaysToDate(date, 1)),
			prevDayProgram = jsp.api.getProgramsByDate(channelId, jsp.addDaysToDate(date, -1));

		jsp.changeVisible(nextArrow, !!nextDayProgram.length);
		jsp.changeVisible(prevArrow, !!prevDayProgram.length);
	}

	function updateProgramsPosition() {
		var focusItem = jsp.byClass("active", programsList),
			COUNT_PER_SCREEN = 7,
			ITEMS_FROM_TOP = 4,
			ITEM_HEIGHT = 56;

		if (!focusItem) {
			return;
		}

		var index = parseInt(focusItem.getAttribute("data-index"), 10) + 1;

		if (index <= COUNT_PER_SCREEN) {
			programsList.scrollTop = 0;
		} else if (index <= COUNT_PER_SCREEN+ITEMS_FROM_TOP) {
			programsList.scrollTop = (index - COUNT_PER_SCREEN) * ITEM_HEIGHT;
		} else {
			programsList.scrollTop = (index - ITEMS_FROM_TOP) * ITEM_HEIGHT;
		}
	}

	function showProgramInfo() {
		var element = document.activeElement,
			channel = jsp.api.getChannelInfo(programsChannel),
			time = parseInt(element.getAttribute("data-time"), 10);

		if (time) updateChannelInfo(channel, new Date(time));
	}

	function channelsListUp() {
		var channel = jsp.api.getChannelInfo(document.activeElement.id),
			newChannel = jsp.api.getChannelInfoByNumber(channel.number-1);

		if (newChannel) loadPrograms(newChannel.id);
		updateChannelsScroll();
	}

	function channelsListDown() {
		var channel = jsp.api.getChannelInfo(document.activeElement.id),
			newChannel = jsp.api.getChannelInfoByNumber(channel.number+1);

		if (newChannel) loadPrograms(newChannel.id);
		updateChannelsScroll();
	}

	function programsNextDay() {
		needFocusToProgram = true;
		renderPrograms(programsChannel, jsp.addDaysToDate(programsDate, 1));
		needFocusToProgram = false;
	}

	function programsPrevDay() {
		needFocusToProgram = true;
		renderPrograms(programsChannel, jsp.addDaysToDate(programsDate, -1));
		needFocusToProgram = false;
	}

	function nextProgram() {
		var el = document.activeElement;
		if (el && el.nextSibling) {
			focusToProgramItem(el.nextSibling);
		}
	}

	function prevProgram() {
		var el = document.activeElement;
		if (el && el.previousSibling) {
			focusToProgramItem(el.previousSibling);
		}
	}

	function focusToProgramItem(item) {
		clearTimeout(updateInfoTimer);
		item.focus();
		updateProgramsScrollBar();
		updateInfoTimer = setTimeout(function (){
			showProgramInfo();
		}, 300);
	}

	function focusToChannel() {
		jsp.byClass("active", channelsTag).focus();
	}

	function focusToProgram() {
		if(renderProgramsTimer !== null) {
			needFocusToProgram = true;
			return;
		}

		var focusItem = jsp.byClass("active", programsList);

		if (focusItem) {
			focusItem.focus();
		} else if (programsList.children[0]) {
			programsList.children[0].focus();
		}
		updateProgramsScrollBar();
	}

	/* ------------------------- SCROLL PROGRESS --------------------------- */

	function updateChannelsScroll() {
		updateScroll(channelsList, channelsScrollBtn);
	}

	function updateProgramsScrollBar() {
		updateScroll(programsList, programsScrollBtn);
	}

	function updateScroll(list, scroll) {
		var SCROLL_MARGIN = 8,
			listHeight = list.offsetHeight,
			scrollHeight = list.scrollHeight;

		if (listHeight < scrollHeight) {
			var btnHeight = (listHeight / scrollHeight) * listHeight,
				scrollPosition = (list.scrollTop / (scrollHeight - listHeight)) * (listHeight - btnHeight);

			scroll.style.height = (btnHeight - SCROLL_MARGIN) + "px";
			scroll.style.top = scrollPosition + "px";
			jsp.show(scroll);
		} else {
			jsp.hide(scroll);
		}
	}

	/* --------------------------- VOLUME ----------------------------- */

	function changeVolume(value) {
		setVolume(jsp.duneapi.getVolume() + value);
	}

	function setVolume(val) {
		var value = val;
		value = Math.max(value, 0);
		value = Math.min(value, 100);
		jsp.duneapi.setVolume(value);
	}

	function toggleMute(){
		if(jsp.duneapi.isMuteEnabled()) {
			jsp.duneapi.disableMute();
		} else {
			jsp.duneapi.enableMute();
		}
	}

	/* -------------------------- EVENT HANDLERS --------------------------- */

	function keyListener(e) {
		var element = document.activeElement,
			action;

		e.preventDefault();

		switch (e.keyCode) {
		case jsp.keyCode.enter:
			action = element.getAttribute("data-enter");
			break;

		case jsp.keyCode.popUpMenu:
			action = element.getAttribute("data-popupmenu");
			break;

		case jsp.keyCode.info:
			showCurrentChannelInfo();
			toggleChannelInfo();
			break;

		case jsp.keyCode.down:
			action = element.getAttribute("data-down");
			break;

		case jsp.keyCode.up:
			action = element.getAttribute("data-up");
			break;

		case jsp.keyCode.prev:
			action = element.getAttribute("data-prev");
			break;

		case jsp.keyCode.next:
			action = element.getAttribute("data-next");
			break;

		case jsp.keyCode.left:
			action = element.getAttribute("data-left");
			break;

		case jsp.keyCode.right:
			action = element.getAttribute("data-right");
			break;

		case jsp.keyCode["return"]:
			if (jsp.isVisible(channelsTag)) {
				hideChannels();
			}
			break;

		case jsp.keyCode.pPlus:
			changeChannelUp();
			break;
		case jsp.keyCode.pMinus:
			changeChannelDown();
			break;

		case jsp.keyCode.vPlus:
			changeVolume(6);
			break;
		case jsp.keyCode.vMinus:
			changeVolume(-5);
			break;
		case jsp.keyCode.mute:
			toggleMute();
			break;
		}

		if (action && jsp.view[action]) {
			jsp.view[action]();
		}
	}

	/* -------------------------- PUBLIC METHODS --------------------------- */

	return {
		init: init,
		focusToProgram: focusToProgram,
		openChannel: openChannel,
		channelsListUp: channelsListUp,
		channelsListDown: channelsListDown,
		changeChannelUp: changeChannelUp,
		changeChannelDown: changeChannelDown,
		showChannels: showChannels,
		hideChannels: hideChannels,
		programsNextDay: programsNextDay,
		programsPrevDay: programsPrevDay,
		prevProgram: prevProgram,
		nextProgram: nextProgram,
		focusToChannel: focusToChannel
	};
})();